/***********************************************************************
 *
 * Copyright (C) 2009 Graeme Gott <graeme@gottcode.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include "board.h"

#include "cell.h"
#include "puzzle.h"

#include <QFrame>
#include <QGridLayout>
#include <QLabel>
#include <QPainter>
#include <QSettings>

/*****************************************************************************/

Board::Board(QWidget* parent)
: QWidget(parent), m_notes_mode(false), m_finished(false) {
	QGridLayout* layout = new QGridLayout(this);
	layout->setMargin(0);
	layout->setSpacing(0);

	// Create cells
	for (int i = 0; i < 9; ++i) {
		int col = (i % 3) * 3;
		int max_col = col + 3;
		int row = (i / 3) * 3;
		int max_row = row + 3;

		Frame* box = new Frame(this);
		box->setBackgroundRole(QPalette::Dark);
		layout->addWidget(box, row / 3, col / 3);

		QGridLayout* box_layout = new QGridLayout(box);
		box_layout->setMargin(2);
		box_layout->setSpacing(1);
		for (int r = row; r < max_row; ++r) {
			for (int c = col; c < max_col; ++c) {
				Cell* cell = new Cell(c, r, this, box);
				box_layout->addWidget(cell, r - row, c - col);
				m_cells[c][r] = cell;
			}
		}
	}

	// Create success message
	m_message = new QLabel(this);
	QFontMetrics metrics(QFont("Sans", 24));
	int width = metrics.width(tr("Success"));
	int height = metrics.height();
	QPixmap success(QSize(width + height, height * 2));
	success.fill(QColor(0, 0, 0, 0));
	{
		QPainter painter(&success);

		painter.setPen(Qt::NoPen);
		painter.setBrush(QColor(0, 0, 0, 200));
		painter.setRenderHint(QPainter::Antialiasing, true);
		painter.drawRoundedRect(0, 0, width + height, height * 2, 10, 10);

		painter.setFont(QFont("Sans", 24));
		painter.setPen(Qt::white);
		painter.setRenderHint(QPainter::TextAntialiasing, true);
		painter.drawText(height / 2, height / 2 + metrics.ascent(), tr("Success"));
	}
	m_message->setPixmap(success);
	m_message->hide();
	layout->addWidget(m_message, 0, 0, 3, 3, Qt::AlignCenter);

	// Load current puzzle
	QSettings settings;
	int seed = settings.value("Current/Seed", 0).toInt();
	if (seed > 0) {
		newPuzzle(seed, true);
		for (int r = 0; r < 9; ++r) {
			for (int c = 0; c < 9; ++c) {
				m_cells[c][r]->loadValue();
			}
		}
		if (settings.contains("Current/Active")) {
			QStringList cell = settings.value("Current/Active").toString().split("x");
			if (cell.count() == 2) {
				m_cells[cell[0].toInt()][cell[1].toInt()]->setFocus();
			}
		}
	} else {
		newPuzzle();
	}
}

/*****************************************************************************/

void Board::newPuzzle(int seed, bool load) {
	if (seed <= 0) {
		qsrand(time(0));
		seed = qrand();
	}

	window()->setWindowFilePath(QString::number(seed));
	if (!load) {
		QSettings settings;
		settings.remove("Current");
		settings.setValue("Current/Seed", seed);
	}

	showWrong(false);
	m_finished = false;
	m_message->hide();

	m_puzzle = new Puzzle(seed);
	for (int r = 0; r < 9; ++r) {
		for (int c = 0; c < 9; ++c) {
			m_cells[c][r]->setPuzzle(m_puzzle);
		}
	}
}

/*****************************************************************************/

void Board::checkFinished() {
	m_finished = true;
	for (int r = 0; r < 9; ++r) {
		for (int c = 0; c < 9; ++c) {
			m_finished = m_finished && m_cells[c][r]->isCorrect();
		}
	}

	if (m_finished) {
		for (int r = 0; r < 9; ++r) {
			for (int c = 0; c < 9; ++c) {
				m_cells[c][r]->clearFocus();
				m_cells[c][r]->setFocusPolicy(Qt::NoFocus);
			}
		}
		QSettings().remove("Current");
		m_message->show();
		update();
	}
}

/*****************************************************************************/

void Board::showWrong(bool show) {
	for (int r = 0; r < 9; ++r) {
		for (int c = 0; c < 9; ++c) {
			m_cells[c][r]->showWrong(show);
		}
	}
}

/*****************************************************************************/

void Board::moveFocus(int column, int row, int xdelta, int ydelta) {
	xdelta = qBound(-1, xdelta, 2);
	ydelta = qBound(-1, ydelta, 2);
	Q_ASSERT(xdelta != ydelta);

	do {
		if (column + xdelta < 0) {
			column = 9;
		} else if (column + xdelta > 8) {
			column = -1;
		}
		column += xdelta;

		if (row + ydelta < 0) {
			row = 9;
		} else if (row + ydelta > 8) {
			row = -1;
		}
		row += ydelta;
	} while (m_cells[column][row]->focusPolicy() == Qt::NoFocus);

	m_cells[column][row]->setFocus();
}

/*****************************************************************************/

void Board::setMode(int mode) {
	m_notes_mode = mode;
	QSettings().setValue("Mode", (m_notes_mode ? "Pencil" : "Pen"));
}

/*****************************************************************************/
