/***********************************************************************
 *
 * Copyright (C) 2009 Graeme Gott <graeme@gottcode.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include "cell.h"

#include "board.h"
#include "puzzle.h"

#include <QKeyEvent>
#include <QPainter>
#include <QResizeEvent>
#include <QSettings>
#include <QSizePolicy>

/*****************************************************************************/

namespace {
	int cell_size = 0;
	int pen_size = 1;
	int pencil_size = 1;
}

/*****************************************************************************/

Cell::Cell(int column, int row, Board* board, QWidget* parent)
: Frame(parent), m_column(column), m_row(row), m_value(0), m_wrong(false), m_given(false), m_board(board), m_puzzle(0) {
	for (int i = 0; i < 9; ++i) {
		m_notes[i] = false;
	}
	setMinimumSize(35, 35);
}

/*****************************************************************************/

bool Cell::isCorrect() const {
	Q_ASSERT(m_puzzle != 0);
	return m_value == m_puzzle->value(m_column, m_row);
}

/*****************************************************************************/

void Cell::setHintVisible(bool visible) {
	if (visible) {
		if (m_value) {
			Q_ASSERT(m_puzzle != 0);
			m_wrong = m_puzzle->value(m_column, m_row) == m_value;
		}
	} else {
		m_wrong = false;
	}
}

/*****************************************************************************/

void Cell::setPuzzle(Puzzle* puzzle) {
	Q_ASSERT(puzzle != 0);
	m_puzzle = puzzle;

	m_conflicts.clear();
	for (int i = 0; i < 9; ++i) {
		m_notes[i] = false;
	}
	m_value = m_puzzle->given(m_column, m_row);
	m_given = (m_value > 0);
	updateFont();

	QFont f = font();
	if (m_given) {
		setBackgroundRole(QPalette::Window);
		setForegroundRole(QPalette::WindowText);
		setFocusPolicy(Qt::NoFocus);
		f.setBold(true);
	} else {
		setBackgroundRole(QPalette::Base);
		setForegroundRole(QPalette::Text);
		setFocusPolicy(Qt::StrongFocus);
		f.setBold(false);
	}
	setFont(f);
}

/*****************************************************************************/

void Cell::loadValue() {
	QString key(QString("Current/%1x%2").arg(m_column).arg(m_row));
	if (!m_given) {
		QVariant value = QSettings().value(key);
		if (value.type() == QVariant::String) {
			toggleValue(qBound(1, value.toInt(), 10));
		} else if (value.type() == QVariant::StringList) {
			QStringList notes = value.toStringList();
			notes.removeFirst();
			foreach (QString note, notes) {
				toggleNote(qBound(1, note.toInt(), 10));
			}
		}
	} else {
		QSettings().remove(key);
	}
}

/*****************************************************************************/

void Cell::showWrong(bool show) {
	m_wrong = show ? !isCorrect() : false;
	update();
}

/*****************************************************************************/

void Cell::focusInEvent(QFocusEvent* event) {
	if (!m_given) {
		QSettings().setValue("Current/Active", QString("%1x%2").arg(m_column).arg(m_row));
		setBackgroundRole(QPalette::Highlight);
		setForegroundRole(QPalette::HighlightedText);
	}
	Frame::focusInEvent(event);
}

/*****************************************************************************/

void Cell::focusOutEvent(QFocusEvent* event) {
	if (!m_given) {
		setBackgroundRole(QPalette::Base);
		setForegroundRole(QPalette::Text);
	}
	Frame::focusOutEvent(event);
}

/*****************************************************************************/

void Cell::keyPressEvent(QKeyEvent* event) {
	Q_ASSERT(!m_given);
	switch (event->key()) {
	case Qt::Key_1:
	case Qt::Key_2:
	case Qt::Key_3:
	case Qt::Key_4:
	case Qt::Key_5:
	case Qt::Key_6:
	case Qt::Key_7:
	case Qt::Key_8:
	case Qt::Key_9:
		if (m_board->notesMode()) {
			toggleNote(event->key() - Qt::Key_0);
		} else {
			toggleValue(event->key() - Qt::Key_0);
		}
		m_board->showWrong(false);
		update();
		save();
		break;
	case Qt::Key_Left:
		m_board->moveFocus(m_column, m_row, -1, 0);
		break;
	case Qt::Key_Right:
		m_board->moveFocus(m_column, m_row, 1, 0);
		break;
	case Qt::Key_Up:
		m_board->moveFocus(m_column, m_row, 0, -1);
		break;
	case Qt::Key_Down:
		m_board->moveFocus(m_column, m_row, 0, 1);
		break;
	default:
		break;
	}
	Frame::keyPressEvent(event);
}

/*****************************************************************************/

void Cell::paintEvent(QPaintEvent* event) {
	Frame::paintEvent(event);

	QPainter painter(this);

	if (m_value) {
		QColor color = palette().color(foregroundRole());
		if (m_wrong) {
			color = Qt::blue;
		} else if (!m_conflicts.isEmpty()) {
			color = Qt::red;
		}
		painter.setPen(color);

		painter.drawText(rect(), Qt::AlignCenter, QString::number(m_value));
	} else {
		painter.setPen(palette().color(foregroundRole()));

		int w = (width() - 8) / 3;
		int h = (height() - 8) / 3;
		for (int i = 0; i < 9; ++i) {
			int c = i % 3;
			int r = i / 3;
			if (m_notes[i]) {
				painter.drawText(QRect(c * w + 4, r * h + 4, w, h), Qt::AlignCenter, QString::number(i + 1));
			}
		}
	}
}

/*****************************************************************************/

void Cell::resizeEvent(QResizeEvent* event) {
	int size = qMin(event->size().width(), event->size().height());
	if (cell_size != size) {
		cell_size = size;
		pen_size = size - 8;
		pencil_size = pen_size / 3;
	}
	updateFont();
}

/*****************************************************************************/

void Cell::toggleNote(int note) {
	m_value = 0;
	updateFont();

	foreach (Cell* cell, m_conflicts) {
		cell->m_conflicts.removeOne(this);
		cell->update();
	}
	m_conflicts.clear();

	m_notes[note - 1] = !m_notes[note - 1];
}

/*****************************************************************************/

void Cell::toggleValue(int value) {
	for (int i = 0; i < 9; ++i) {
		m_notes[i] = false;
	}
	m_value = (value != m_value) ? value : 0;
	updateFont();

	// Check for conflicts
	foreach (Cell* cell, m_conflicts) {
		cell->m_conflicts.removeOne(this);
		cell->update();
	}
	m_conflicts.clear();

	for (int c = 0; c < 9; ++c) {
		checkConflict(m_board->cell(c, m_row));
	}

	for (int r = 0; r < 9; ++r) {
		checkConflict(m_board->cell(m_column, r));
	}

	int col = (m_column / 3) * 3;
	int max_col = col + 3;
	int row = (m_row / 3) * 3;
	int max_row = row + 3;
	for (int r = row; r < max_row; ++r) {
		for (int c = col; c < max_col; ++c) {
			checkConflict(m_board->cell(c, r));
		}
	}

	// Check if game is over
	m_board->checkFinished();
}

/*****************************************************************************/

void Cell::checkConflict(Cell* cell) {
	if (cell != this && !m_conflicts.contains(cell)) {
		if (cell->m_value == m_value) {
			m_conflicts.append(cell);
			cell->m_conflicts.append(this);
			cell->update();
		}
	}
}

/*****************************************************************************/

void Cell::save() {
	QString key(QString("Current/%1x%2").arg(m_column).arg(m_row));
	if (m_value) {
		QSettings().setValue(key, m_value);
	} else {
		QStringList notes;
		for (int i = 0; i < 9; ++i) {
			if (m_notes[i]) {
				notes.append(QString::number(i+1));
			}
		}

		if (!notes.isEmpty()) {
			notes.prepend("Notes");
			QSettings().setValue(key, notes);
		} else {
			QSettings().remove(key);
		}
	}
}

/*****************************************************************************/

void Cell::updateFont() {
	QFont f = font();
	f.setPixelSize(m_value ? pen_size : pencil_size);
	setFont(f);
}

/*****************************************************************************/
