/***********************************************************************
 *
 * Copyright (C) 2009 Graeme Gott <graeme@gottcode.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include "puzzle.h"

#include "dancing_links.h"

#include <QList>

/*****************************************************************************/

Puzzle::Puzzle(unsigned int seed) {
	qsrand(seed);

	// Fill solution grid
	while (!createSolution()) {
	}

	// Find givens
	createGivens();

	// Remove any extra givens
	reduceGivens();
}

/*****************************************************************************/

bool Puzzle::createSolution() {
	// Create list of initial values
	QList<int> initial;
	for (int i = 1; i < 10; ++i) {
		initial.append(i);
	}

	// Fill possible values into all cells
	QList<int> cells[9][9];
	for (int r = 0; r < 9; ++r) {
		for (int c = 0; c < 9; ++c) {
			cells[c][r] = initial;
		}
	}

	// Cycle through all cells, finding a value for each cell
	for (int r = 0; r < 9; ++r) {
		for (int c = 0; c < 9; ++c) {
			QList<int>& cell = cells[c][r];

			// Abort if seed produces invalid solution
			if (cell.isEmpty()) {
				return false;
			}

			// Randomly select value from possiblities
			int pos = qrand() % cell.count();
			int value = cell.takeAt(pos);

			// Place value into solution
			m_solution[c][r] = value;

			// Remove all instances of value in row
			for (int row = 0; row < 9; ++row) {
				cells[c][row].removeOne(value);
			}

			// Remove all instances of value in column
			for (int col = 0; col < 9; ++col) {
				cells[col][r].removeOne(value);
			}

			// Remove all instances of value in box
			int box_row = (r / 3) * 3;
			int box_col = (c / 3) * 3;
			for (int row = box_row; row < box_row + 3; ++row) {
				for (int col = box_col; col < box_col + 3; ++col) {
					cells[col][row].removeOne(value);
				}
			}
		}
	}

	return true;
}

/*****************************************************************************/

void Puzzle::createGivens() {
	// Initialize givens
	for (int r = 0; r < 9; ++r) {
		for (int c = 0; c < 9; ++c) {
			m_givens[c][r] = 0;
		}
	}

	int givens = 0;
	bool done = false;
	do {
		// Find cells
		int x1 = qrand() % 8;
		int x2 = 8 - x1;
		int y1 = qrand() % 8;
		int y2 = 8 - y1;

		// Check to make sure they aren't already visible
		if (m_givens[x1][y1] != 0 || m_givens[x2][y2] != 0) {
			continue;
		}

		// Copy givens to puzzle
		m_givens[x1][y1] = m_solution[x1][y1];
		m_givens[x2][y2] = m_solution[x2][y2];
		givens += (x1 != x2 || y1 != y2) ? 2 : 1;

		// Solve puzzle
		if (givens >= 17) {
			done = isUnique();
		}
	} while (!done);
}

/*****************************************************************************/

void Puzzle::reduceGivens() {
	for (int c = 0; c < 9; ++c) {
		for (int r = 0; r < 9; ++r) {
			int given1 = m_givens[c][r];
			int given2 = m_givens[8-c][8-r];
			m_givens[c][r] = 0;
			m_givens[8-c][8-r] = 0;
			if (!isUnique()) {
				m_givens[c][r] = given1;
				m_givens[8-c][8-r] = given2;
			}
		}
	}
}

/*****************************************************************************/

bool Puzzle::isUnique() {
	QList<int> initial;
	for (int i = 0; i < 9; ++i) {
		initial.append(i);
	}

	DLX::Matrix matrix(324);
	for (int r = 0; r < 9; ++r) {
		for (int c = 0; c < 9; ++c) {
			int given = m_givens[c][r];
			QList<int> values = (given == 0) ? initial : initial.mid(given - 1, 1);
			foreach (int value, values) {
				matrix.addRow();
				matrix.addElement(r * 9 + c);
				matrix.addElement(r * 9 + value + 81);
				matrix.addElement(c * 9 + value + 162);
				matrix.addElement((3 * (r / 3) + (c / 3)) * 9 + value + 243);
			}
		}
	}

	return (matrix.search(2) == 1);
}

/*****************************************************************************/
