/***********************************************************************
 *
 * Copyright (C) 2009 Graeme Gott <graeme@gottcode.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include "window.h"

#include "board.h"

#include <QApplication>
#include <QButtonGroup>
#include <QHBoxLayout>
#include <QInputDialog>
#include <QMenuBar>
#include <QMessageBox>
#include <QPushButton>
#include <QSettings>
#include <QStyle>
#include <QVBoxLayout>

/*****************************************************************************/

Window::Window()
: m_board_offset(0) {
#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
	setWindowIcon(QIcon(":/simsu.png"));
#endif

	QWidget* contents = new QWidget(this);
	setCentralWidget(contents);

	// Create board
	m_board = new Board(contents);

	// Create buttons
	m_mode_buttons = new QButtonGroup(this);
	connect(m_mode_buttons, SIGNAL(buttonClicked(int)), m_board, SLOT(setMode(int)));

	QPushButton* pen_button = new QPushButton(tr("Pen"), contents);
	pen_button->setCheckable(true);
	pen_button->setAutoDefault(false);
	pen_button->setFocusPolicy(Qt::NoFocus);
	m_mode_buttons->addButton(pen_button, 0);

	QPushButton* pencil_button = new QPushButton(tr("Pencil"), contents);
	pencil_button->setCheckable(true);
	pencil_button->setAutoDefault(false);
	pencil_button->setFocusPolicy(Qt::NoFocus);
	m_mode_buttons->addButton(pencil_button, 1);

	m_mode_buttons->button(QSettings().value("Mode") == "Pencil")->setChecked(true);
	m_board->setMode(m_mode_buttons->checkedId());

	QHBoxLayout* buttons_layout = new QHBoxLayout;
	buttons_layout->addWidget(pen_button);
	buttons_layout->addStretch();
	buttons_layout->addWidget(pencil_button);

	// Layout window
	QVBoxLayout* layout = new QVBoxLayout(contents);
	layout->setMargin(5);
	layout->addStretch();
	layout->addLayout(buttons_layout);

	// Create menus
	QMenu* menu = menuBar()->addMenu(tr("&Game"));
	menu->addAction(tr("&New"), this, SLOT(newGame()), tr("Ctrl+N"));
	menu->addAction(tr("&Choose..."), this, SLOT(chooseGame()));
	menu->addSeparator();
	menu->addAction(tr("&Switch Modes"), this, SLOT(toggleMode()), tr("S"));
	menu->addAction(tr("Check &Answers"), m_board, SLOT(showWrong()), tr("C"));
	menu->addSeparator();
	menu->addAction(tr("&Quit"), qApp, SLOT(quit()), tr("Ctrl+Q"));

	menu = menuBar()->addMenu(tr("&Help"));
	menu->addAction(tr("&About"), this, SLOT(about()));
	menu->addAction(tr("About &Qt"), qApp, SLOT(aboutQt()));

	// Restore size and position
	m_board_offset = menuBar()->sizeHint().height() + pen_button->sizeHint().height() + 10;
	setMinimumSize(345, 345 + m_board_offset);
	restoreGeometry(QSettings().value("Geometry").toByteArray());
}

/*****************************************************************************/

void Window::closeEvent(QCloseEvent* event) {
	QSettings().setValue("Geometry", saveGeometry());
	QMainWindow::closeEvent(event);
}

/*****************************************************************************/

void Window::resizeEvent(QResizeEvent* event) {
	QMainWindow::resizeEvent(event);

	QRect region = childrenRect();
	region.setHeight(region.height() - m_board_offset);
	int size = qMin(region.width(), region.height()) - 10;

	QRect rect = QStyle::alignedRect(Qt::LeftToRight, Qt::AlignCenter, QSize(size, size), region);
	m_board->setGeometry(rect);
}

/*****************************************************************************/

void Window::newGame() {
	if (abortGame()) {
		m_board->newPuzzle();
	}
}

/*****************************************************************************/

void Window::chooseGame() {
	if (abortGame()) {
		bool ok;
		int seed = QInputDialog::getInteger(this, tr("Choose Game"), tr("Enter a game number:"), QSettings().value("Current/Seed", 1).toInt(), 1, 2147483647, 1, &ok);
		if (ok) {
			m_board->newPuzzle(seed);
		}
	}
}

/*****************************************************************************/

bool Window::abortGame() {
	if (!m_board->isFinished()) {
		return QMessageBox::question(this, tr("Question"), tr("Do you want to end this game to start a new one?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes;
	} else {
		return true;
	}
}

/*****************************************************************************/

void Window::about() {
	QMessageBox::about(this, tr("About Simsu"), tr("<center><big><b>Simsu 1.0.0</b></big><br/>A basic Sudoku game<br/><small>Copyright &copy; 2009 Graeme Gott</small></center>"));
}

/*****************************************************************************/

void Window::toggleMode() {
	m_mode_buttons->button(!m_mode_buttons->checkedId())->click();
}

/*****************************************************************************/
